#SLIP-0013 : Authentication using deterministic hierarchy

```
Number:  SLIP-0013
Title:   Authentication using deterministic hierarchy
Type:    Standard
Status:  Draft
Authors: Pavol Rusnak <stick@satoshilabs.com>
Created: 2015-03-12
```

##Abstract

This document describes a method that is used for authenticating
to various services such as websites or remote shells using a determinstic
hierarchy.

##Motivation

Using Deterministic Hierarchy for authenticating into systems is ideal,
because the same concepts of easy backup that relate to backing up
deterministic wallets can be applied to backing up user identities.

##Service Identity

Let's introduce the service identity. It consists of two elements:

a) RFC 3986 URI `proto://[user@]host[:port][/path]`

Examples:

- https://example.com
- ftp://public@example.com/pub
- ssh://root@example.com:2222

b) index (32-bit unsigned integer)

The index is used so one can generate more keys corresponding to the same URI.

##HD Structure

1. Let's concatenate the little endian representation of index with the URI.

2. Compute the SHA256 hash of the result.

3. Let's take first 128 bits of the hash and split it into four 32-bit numbers A, B, C, D.

4. Set highest bits of numbers A, B, C, D to 1.

5. Derive the HD node m/13'/A'/B'/C'/D' according to BIP32.

##Challenge - Response

Service issues the challenge consisting of three parts:

a) service identity described above (e.g. https://example.com 0)

b) hidden challenge
   - random bytes sequence of maximum length 64
   - this won't be shown to the user

c) visual challenge
   - arbitrary string of text of maximum length 64
   - this will be shown to the user and we recommend using timestamp in `YYYY-MM-DD HH:MM:SS` format or similar

Signer takes this data and computes the private key according to section HD Structure.
Then it concatenates sha256 hashes of challenge hidden and challenge visual and
signs the result using the standard Bitcoin message signing.
Finally, the signature is returned together with the node public key and node address.

It's up to service operator to take this message and react in three possible ways:

1. signature is invalid or not present -> show error to user
2. signature is valid, address/public key seen for the first time -> create user account
3. signature is valid, address/public key known -> login to user account

##References

- [BIP-0032: Hierarchical Deterministic Wallets](https://github.com/bitcoin/bips/blob/master/bip-0032.mediawiki)
- [BIP-0043: Purpose Field for Deterministic Wallets](https://github.com/bitcoin/bips/blob/master/bip-0043.mediawiki)
- [RFC 3986: Uniform Resource Identifier (URI): Generic Syntax](https://tools.ietf.org/html/rfc3986)
